import datetime
import os
import re

import requests
from pontis.core.entry import construct_entry
from pontis.core.utils import currency_pair_to_key


def fetch_binance(assets):
    PUBLISHER_PREFIX = os.environ.get("PUBLISHER_PREFIX")
    publisher = PUBLISHER_PREFIX + "-cryptowatch-coinbase-pro"

    response = requests.get("https://api.cryptowat.ch/markets/coinbase-pro", timeout=5)

    entries = []

    for asset in assets:
        if asset["type"] != "SPOT":
            print(f"Skipping Cryptowatch for non-spot asset {asset}")
            continue

        pair = asset["pair"]
        result = [e for e in response.json()["result"] if e["pair"] == "".join(pair)]
        if len(result) == 0:
            print(f"No entry found for {key} from Cryptowatch")
            continue

        assert (
            len(result) == 1
        ), f"Found more than one matching entries for Cryptowatch response and price pair {pair}"

        result = response.json()

        timestamp = int(result["timestamp"])
        price = float(result["last"])
        price_int = int(price * (10 ** asset["decimals"]))
        key = currency_pair_to_key(*pair)

        print(f"Fetched price {price} for {'/'.join(pair)} from Bitstamp")

        entries.append(
            construct_entry(
                key=key,
                value=price_int,
                timestamp=timestamp,
                publisher=publisher,
            )
        )

    return entries



from pontis.core.const import NETWORK, ORACLE_CONTROLLER_ADDRESS
from pontis.core.utils import str_to_felt
from starknet_py.contract import Contract
from starknet_py.net import Client

MAX_FEE = 0
DEFAULT_N_RETRIES = 1


class PontisClient:
    def __init__(
        self, network=None, oracle_controller_address=None, max_fee=None, n_retries=None
    ):
        if network is None:
            network = NETWORK
        if oracle_controller_address is None:
            oracle_controller_address = ORACLE_CONTROLLER_ADDRESS

        self.network = network
        self.oracle_controller_address = oracle_controller_address
        self.oracle_controller_contract = None

        self.max_fee = MAX_FEE if max_fee is None else max_fee
        self.n_retries = DEFAULT_N_RETRIES if n_retries is None else n_retries

    async def fetch_oracle_controller_contract(self):
        if self.oracle_controller_contract is None:
            self.oracle_controller_contract = await Contract.from_address(
                self.oracle_controller_address,
                Client(self.network, n_retries=self.n_retries),
            )

    async def get_decimals(self, key):
        await self.fetch_oracle_controller_contract()

        if type(key) == str:
            key = str_to_felt(key)
        elif type(key) != int:
            raise AssertionError(
                "Key must be string (will be converted to felt) or integer"
            )

        response = await self.oracle_controller_contract.functions["get_decimals"].call(
            key
        )

        return response.decimals

    async def get_value(self, key, aggregation_mode):
        await self.fetch_oracle_controller_contract()

        if type(key) == str:
            key = str_to_felt(key)
        elif type(key) != int:
            raise AssertionError(
                "Key must be string (will be converted to felt) or integer"
            )

        response = await self.oracle_controller_contract.functions["get_value"].call(
            key, aggregation_mode
        )

        return response.value, response.last_updated_timestamp

    async def get_entries(self, key):
        await self.fetch_oracle_controller_contract()

        if type(key) == str:
            key = str_to_felt(key)
        elif type(key) != int:
            raise AssertionError(
                "Key must be string (will be converted to felt) or integer"
            )

        response = await self.oracle_controller_contract.functions["get_entries"].call(
            key
        )

        return response.entries



import datetime
import os
import re

import requests
from pontis.core.entry import construct_entry


def fetch_binance(assets):
    PUBLISHER_PREFIX = os.environ.get("PUBLISHER_PREFIX")
    publisher = PUBLISHER_PREFIX + "-binance"

    base_url = "https://dapi.binance.com/dapi/v1"
    response = requests.get(base_url + "/premiumIndex", timeout=20)

    entries = []

    # Don't fetch spot data because Binance only has crypto/crypto spot price pairs

    for asset in assets:
        if asset["type"] != "FUTURE":
            print(f"Skipping Binance for non-futures asset {asset}")
            continue

        pair = asset["pair"]

        result = [
            e
            for e in response.json()
            if re.match(rf"{pair[0]}{pair[1]}_[0-9]+", e["symbol"])
        ]
        if len(result) == 0:
            print(f"No entry found for {asset['type']} {'/'.join(pair)} from Binance")
            continue

        for future in result:
            timestamp = int(future["time"] / 1000)
            price = float(future["markPrice"])
            price_int = int(price * (10 ** asset["decimals"]))

            future_expiration_date = int(
                datetime.datetime.strptime(
                    future["symbol"],
                    f"{pair[0]}{pair[1]}_%y%m%d",
                ).strftime("%Y%m%d")
            )
            key = f"{pair[0]}/{pair[1]}-{future_expiration_date}".lower()

            print(f"Fetched futures price {price} for {key} from Binance")

            entries.append(
                construct_entry(
                    key=key,
                    value=price_int,
                    timestamp=timestamp,
                    publisher=publisher,
                )
            )

    return entries



import datetime
import os

import requests
from pontis.core.entry import construct_entry
from pontis.core.utils import currency_pair_to_key


def fetch_coinmarketcap(assets):
    PUBLISHER_PREFIX = os.environ.get("PUBLISHER_PREFIX")
    publisher = PUBLISHER_PREFIX + "-coinmarketcap"
    COINMARKETCAP_KEY = os.environ.get("COINMARKETCAP_KEY")
    headers = {
        "X-CMC_PRO_API_KEY": COINMARKETCAP_KEY,
        "Accepts": "application/json",
    }

    url = "https://pro-api.coinmarketcap.com/v1/cryptocurrency/quotes/latest"

    entries = []

    for asset in assets:
        if asset["type"] != "SPOT":
            print(f"Skipping Coinmarketcap for non-spot asset {asset}")
            continue

        pair = asset["pair"]

        key = currency_pair_to_key(*pair)
        parameters = {"symbol": pair[0], "convert": pair[1]}

        response = requests.get(url, headers=headers, params=parameters, timeout=10)
        response.raise_for_status()
        price = response.json()["data"][pair[0]]["quote"][pair[1]]["price"]
        timestamp = int(
            datetime.datetime.strptime(
                response.json()["data"][pair[0]]["quote"][pair[1]]["last_updated"],
                "%Y-%m-%dT%H:%M:%S.%f%z",
            ).timestamp()
        )
        price_int = int(price * (10 ** asset["decimals"]))

        print(f"Fetched price {price} for {key} from Coinmarketcap")

        entries.append(
            construct_entry(
                key=key,
                value=price_int,
                timestamp=timestamp,
                publisher=publisher,
            )
        )

    return entries



import datetime
import hmac
import os
import re
import time

import requests
from pontis.core.entry import construct_entry
from pontis.core.utils import currency_pair_to_key


def parse_ftx_spot(asset, data, publisher, timestamp):
    pair = asset["pair"]
    key = currency_pair_to_key(*pair)

    result = [e for e in data if e["name"] == "/".join(pair)]
    if len(result) == 0:
        print(f"No entry found for {'/'.join(pair)} from FTX")
        return

    assert (
        len(result) == 1
    ), f"Found more than one matching entries for FTX response and price pair {pair}"
    price = float(result[0]["price"])
    price_int = int(price * (10 ** asset["decimals"]))

    print(f"Fetched price {price} for {'/'.join(pair)} from FTX")

    return construct_entry(
        key=key,
        value=price_int,
        timestamp=timestamp,
        publisher=publisher,
    )


def parse_ftx_futures(asset, data, publisher, timestamp):
    pair = asset["pair"]
    if pair[1] != "USD":
        print(f"Unable to fetch price from FTX for non-USD derivative {pair}")
        return

    result = [e for e in data if re.match(rf"{pair[0]}-[0-9]+", e["name"])]
    if len(result) == 0:
        print(f"No entry found for {'/'.join(pair)} from FTX")
        return

    entries = []

    for future in result:
        price = float(future["mark"])
        price_int = int(price * (10 ** asset["decimals"]))

        future_expiration_date = int(
            datetime.datetime.strptime(
                future["expiry"],
                "%Y-%m-%dT%H:%M:%S%z",
            ).strftime("%Y%m%d")
        )
        key = f"{pair[0]}/{pair[1]}-{future_expiration_date}".lower()

        print(f"Fetched futures price {price} for {key} from FTX")

        entries.append(
            construct_entry(
                key=key,
                value=price_int,
                timestamp=timestamp,
                publisher=publisher,
            )
        )

    return entries


def generate_ftx_headers(endpoint):
    FTX_API_KEY = os.environ.get("FTX_API_KEY")
    FTX_API_SECRET = os.environ.get("FTX_API_SECRET")

    timestamp = int(time.time() * 1000)
    signature = hmac.new(
        FTX_API_SECRET.encode(),
        (str(timestamp) + "GET" + endpoint).encode("ascii"),
        "sha256",
    ).hexdigest()
    headers = {
        "FTX-KEY": FTX_API_KEY,
        "FTX-SIGN": signature,
        "FTX-TS": str(timestamp),
    }
    return headers


def fetch_ftx(assets):
    PUBLISHER_PREFIX = os.environ.get("PUBLISHER_PREFIX")
    publisher = PUBLISHER_PREFIX + "-ftx"
    base_url = "https://ftx.com/api"

    endpoint = "/markets"
    headers = generate_ftx_headers(endpoint)
    response = requests.get(base_url + endpoint, headers=headers, timeout=10)
    spot_data = response.json()["result"]

    endpoint = "/futures"
    headers = generate_ftx_headers(endpoint)
    response = requests.get(base_url + endpoint, headers=headers, timeout=10)
    future_data = response.json()["result"]

    timestamp = int(time.time())

    entries = []

    for asset in assets:
        if asset["type"] == "SPOT":
            entry = parse_ftx_spot(asset, spot_data, publisher, timestamp)
            if entry is not None:
                entries.append(entry)
            continue
        elif asset["type"] == "FUTURE":
            future_entries = parse_ftx_futures(asset, future_data, publisher, timestamp)
            if len(future_entries) is not None:
                entries.extend(future_entries)
            continue
        else:
            print(f"Unable to fetch FTX for un-supported asset type {asset}")

    return entries



import os

ADMIN_ADDRESS = 0x032611AA4C7610D0B6306ADFFA27BF3E8774D1A27C3E9F93CDC2F7BCFEC0C3D7
ORACLE_CONTROLLER_ADDRESS = (
    0x013BEFE6EDA920CE4AF05A50A67BD808D67EEE6BA47BB0892BEF2D630EAF1BBA
)
PUBLISHER_REGISTRY_ADDRESS = (
    0x07E05E4DEA8A62988D9A06EA47BDAC34C759A413DB5B358E4A3A3D691D9D89E4
)
NETWORK = "testnet"
DEFAULT_AGGREGATION_MODE = 0

if os.environ.get("__PONTIS_STAGING_ENV__") == "TRUE":
    print("Warning: Communicating with staging contracts, not production")
    ORACLE_CONTROLLER_ADDRESS = (
        0x0144B11738F284CA4E6E0B58C71CA814BB025DA724D367550E86AD7925CF21EE
    )
    PUBLISHER_REGISTRY_ADDRESS = (
        0x014385A4C5006834E4407C09E08CD3739FA0221D92CE2FEEC9B830D206723C70
    )
